require 5.002;

use FileHandle;
use ExtUtils::MakeMaker;
use Config;

#
# User Configuration:
#
# Set $install_dir to the directory where the RIB tools should be installed.
#

$install_dir = '/home/olson/nse/RIB/install';

if (! -d $install_dir)
{
    die "Installation directory $install_dir does not exist. Please create it and rerun Makefile.PL\n";
}

$cgi_url = "http://nhse.mcs.anl.gov/home/RIB/cgi-bin";
$install_cgi_bin = "$install_dir/cgi-bin";
$install_bin = "$install_dir/bin";
$install_lib = "$install_dir/lib";
$install_man1 = "$install_dir/man/man1";
$install_man3 = "$install_dir/man/man3";

chop($pwd = `pwd`);
$install_blib_lib = "$pwd/blib/lib";
$install_blib_arch = "$pwd/blib/arch";

@SCRIPTS = qw(add-asset
	      delete-asset
	      gen-catalog
	      init-repository
	      list-assets
	      list-repositories
	      retrieve-metadata
	      test-db-parser
	      test-parser
	      );

@CGI_SCRIPTS = qw(list-assets-cgi
		  repos-admin
		  show-asset-form-action
		 );		

my %opts = ('NAME'	=> 'MTk',
	    'VERSION_FROM' => 'lib/MTk.pm', # finds $VERSION
	    PL_FILES => {
		(map { ("bin/$_.PL", "bin/$_") } @SCRIPTS),
		(map { ("cgi-bin/$_.PL", "cgi-bin/$_") } @CGI_SCRIPTS)
	    },
	    MAN1PODS => {},
	    MAN3PODS => {},
	    EXE_FILES => [(map {"bin/$_"} @SCRIPTS)],
#			  (map {"cgi-bin/$_"} @CGI_SCRIPTS)  ],
	    INSTALLDIRS => site,
	    INSTALLSITEARCH => $install_lib,
	    INSTALLSITELIB => $install_lib,
	    INSTALLBIN => $install_bin,
	    INSTALLSCRIPT => $install_bin,
	    INSTALLMAN1DIR => $install_man1,
	    INSTALLMAN3DIR => $install_man3,
	    clean => {FILES => join(" ",
				    (map {"bin/$_"} @SCRIPTS),
				    (map {"cgi-bin/$_"} @CGI_SCRIPTS),
				    "lib/MTk/Config.pm",
				    "lib/BIDM/Config.pm",
				   )},
);

###################################################
#
# User Configuration:
#
# MTk module configuration
#
# This will have to change if a different database system
# is added. For now, we're just using DBI and mSQL so this needn't
# change.
#
###################################################

#    
# $db_subclass is the subclass of MTk::DB that is to be used for
# MTk
#
$db_subclass = "DBI";

#
# If we're using DBI, $dbi_driver is the driver the DBI module should be
# using.
#

#
# User Configuration:
#
# Set $dbi_host and $dbi_dbname to the name of the database server
# and the name of the database that will be used for MTk.
#

$dbi_driver = 'mSQL';
$dbi_host = 'putnam.mcs.anl.gov';
$dbi_dbname = 'RIB';

###################################################
#
# BIDM module configuration
#
###################################################
    
@bidm_class_path = ('./blib/lib/classes',
		    $install_lib . '/classes');
		      

&write_mtk_config();
&write_bidm_config();

unless ($ExtUtils::MakeMaker::Setup_done++){
    ExtUtils::MakeMaker::full_setup();
    undef &ExtUtils::MakeMaker::full_setup; #safe memory
}
{
    local $SIG{__WARN__} = \&ExtUtils::MakeMaker::warnhandler;

    $mm = MM->new(\%opts);

    #
    # Now we've got a handle on the perl path we should use;
    # hopefully it's the short name
    #

    #
    # Write the scripts
    #
    
#    &gen_scripts($mm->{PERL});

    $mm->flush();
}


#
# Create the MTk config file.
#
sub write_mtk_config
{
    my($config_file, $out);
    
    $config_file = "lib/MTk/Config.pm";
    $out = new FileHandle ">$config_file";
    
    if (!$out)
    {
	die "Cannot open config file $config_file: $!\n";
    }
    
    print "Writing config file $config_file\n";

    print $out <<END;
package MTk;

#
# Config.pm created by Makefile.PL
#

use strict;
use vars qw(\$install_bin \$install_lib \$install_man1 \$install_man3);
use vars qw(\$DBClass);

require MTk::DB::$db_subclass;

\$MTk::install_bin = '$install_bin';
\$MTk::install_lib = '$install_lib';
\$MTk::install_man1 = '$install_man1';
\$MTk::install_man3 = '$install_man3';
\$MTk::cgi_bin_url = '$cgi_url';

\$MTk::DBClass = 'MTk::DB::$db_subclass';

END

    if ($db_subclass eq 'DBI')
    {
	print $out (<<END);
\$MTk::DB::DBI::Driver = '$dbi_driver';
\$MTk::DB::DBI::ServerHost = '$dbi_host';
\$MTk::DB::DBI::DBName = '$dbi_dbname';

END
    }
    
    print $out "\n1;\n";

    $out->close();
}

#
# Create the BIDM config file.
#
sub write_bidm_config
{
    my($config_file, $out, $path);
    
    $config_file = "lib/BIDM/Config.pm";
    $out = new FileHandle ">$config_file";
    
    if (!$out)
    {
	die "Cannot open config file $config_file: $!\n";
    }
    
    print "Writing config file $config_file\n";

    $path = join(" ", @bidm_class_path);

    print $out <<END;
package BIDM;

#
# Config.pm created by Makefile.PL
#

use strict;
use vars qw(\@ClassDefinitionPath);

\@ClassDefinitionPath = qw($path);

1;
END

    $out->close();
}

#
# Create the wrappers for the commands
#

sub gen_scripts
{
    my($perl_path) = @_;
    my($script);

    print "Generating scripts with perl path $perl_path\n";

    for $script (@SCRIPTS)
    {
	system($Config{perlpath}, "bin/$script.PL",  $perl_path,
	       $install_blib_arch, $install_blib_lib, $install_lib);
    }
    for $script (@CGI_SCRIPTS)
    {
	system($Config{perlpath}, "cgi-bin/$script.PL",  $perl_path,
	       $install_blib_arch, $install_blib_lib, $install_lib);
    }
}

# Determine things that should *not* be installed
sub MY::libscan {
    my($self, $path) = @_;
#    return '' if $path =~ m:/classes:;
    return '' if $path =~ m/.(pl|dtd|sgml)$/;
    return '' if $path =~ m:\bCVS/:;
    return '' if $path =~ m/~$/;
    return '' if $path =~ m:/#[^/]*$:;
#    print "libscan: path=$path\n";
    $path;
}

#
# We generate our own .PL processor that passes in the short pathname
# for the perl executable.
#

sub MY::processPL {
    my($self) = shift;
    return "" unless $self->{PL_FILES};
    my(@m, $plfile);
    foreach $plfile (sort keys %{$self->{PL_FILES}}) {
	push @m, "
all :: $self->{PL_FILES}->{$plfile}

$self->{PL_FILES}->{$plfile} :: $plfile
	\$(PERL) -I\$(INST_ARCHLIB) -I\$(INST_LIB) -I\$(PERL_ARCHLIB) -I\$(PERL_LIB)  $plfile '\$(PERL)' '\$(INST_ARCHLIB)' '\$(INST_LIB)' '\$(INSTALLSITEARCH)' '\$(INSTALLSITELIB)' 
";
    }
    join "", @m;
}

sub MY::postamble
{
    my($cgi);
    my($out, $all);

    $out = "\n\INST_CGI = ./blib/cgi-bin\n\n";
    $all = "all ::";

    $out .= <<END;
\$(INST_CGI)/.exists :: /afs/fl.mcs.anl.gov/appl/perl5/lib/sun4-sunos/5.00305/CORE/perl.h
	@\$(MKPATH) \$(INST_CGI)
	@\$(EQUALIZE_TIMESTAMP) $Config{archlibexp}/CORE/perl.h \$(INST_CGI)/.exists
	-@\$(CHMOD) 755 \$(INST_CGI)

END
    
    for $cgi (@CGI_SCRIPTS)
    {
	$all .= " \$(INST_CGI)/$cgi";
	$out .= <<END;
\$(INST_CGI)/$cgi: cgi-bin/$cgi Makefile \$(INST_CGI)/.exists
	\@rm -f \$(INST_CGI)/$cgi
	cp cgi-bin/$cgi \$(INST_CGI)/$cgi

END
    }

    $out .= "$all\n\n";
    $out .= <<END;
    
install ::
	\$(MOD_INSTALL) \\
		\$(INST_CGI) $install_cgi_bin
END
    return $out;
}

sub MY::init_main
{
    my($self, @args) = @_;
    my($inherited) = $self->SUPER::init_main(@args);

    $self->{INST_CGI} ||= $self->catdir($self->curdir,'blib','cgi-bin');

    print "set inst_cgi to $self->{INST_CGI}\n";
    return $inherited;
}
