use Config;
use File::Basename qw(basename dirname);
chdir(dirname($0));

$perl = shift(@ARGV);

$use = "use lib qw(\n\t" . join("\n\t", @ARGV) . ");\n";

($file = basename($0)) =~ s/\.PL$//;
$file =~ s/\.pl$//
	if ($Config{'osname'} eq 'VMS' or
	    $Config{'osname'} eq 'OS2');  # "case-forgiving"


open OUT,">$file" or die "Can't create $file: $!";
chmod(0755, $file);
print "Extracting $file (with variable substitutions)\n";

print OUT <<"!GROK!THIS!";
#!$perl -w

#
# Warning: File automatically generated. Any changes made here
# will be lost.
#

$use

##HeaderEnd

!GROK!THIS!
print OUT <<'!NO!SUBS!';

=head1 NAME

test-parser

=head1 SYNOPSIS

test-parser filename

=head1 DESCRIPTION

Test jacket code for debugging the meta tag parser.

=cut

use strict;

use BIDM;
use BIDM::Webpage;
use LWP::RobotUA;
use MTk;

require wallclock;

#eval {
    &main;
#};

if ($@)
{
    print "$0 failed:\n\t$@\n";
    exit(1);
}


exit(0);

sub main
{
    my($db, $bidm_tagset, $page, $user_agent, $url);

    if (@ARGV == 0)
    {
	$url = 'http://www.netlib.org/scalapack/';
    }
    elsif (@ARGV == 1)
    {
	$url = shift(@ARGV);
    }
    else
    {
	die "Usage: $0\n";
    }

    $db = $MTk::DBClass->new();
    $db or die "Cannot connect to database\n";
    
    $db->open();

    $bidm_tagset = new BIDM::Tagset;

    $bidm_tagset->load_from_db($db, $url);
    
    print "Got tagset $bidm_tagset\n";
    $bidm_tagset->print();
    my $t1 = wallclock();
    my $obj = $bidm_tagset->parse();
    my $t2 = wallclock();

    $obj->walk();

    my $elap = $t2 - $t1;
    print "exiting, elap=$elap\n";
}

package BIDM::RIGObject;
sub walk
{
    my($self) = @_;
    print "RIG walk $self\n";
}

package BIDM::RIGObject::Asset;
sub walk
{
    my($self) = @_;
    my($name, $k, $contact);
    print "Asset walk $self\n";

    $name = $self->TitleLine();
    if (defined($name))
    {
	print "name=$name\n";
    }

    for $k ($self->Keyword())
    {
	print "Keyword: $k\n";
    }
    print "Abstract:\n---\n", $self->Abstract(), "\n---\n";

    my $web = $self->WebPage();
    print "web=$web\n";

    if (ref($web))
    {
	my $url = $web->href();
	print "web url $url\n";
    }

    my @contacts = $self->ContactIs();

    for $contact (@contacts)
    {
	print "Contact: \n";
	my($name, $address, $email, $fax, $telephone);
	$name = $contact->Name();
	$address= $contact->Address();
	$email = $contact->Email();
	$fax = $contact->Fax();
	$telephone = $contact->Telephone();
	print "$name\n$address\n$email\n$fax\n$telephone\n";
#	$contact->print();
    }
}

package BIDM::RIGObject::Organization;
sub walk
{
    my($self) = @_;
    print "Organization walk $self\n";
}

package BIDM::RIGObject::Library;
sub walk
{
    my($self) = @_;
    print "Library walk $self\n";
}
!NO!SUBS!
