package MTk::DB::DBI;

=head1 NAME

MTk::DB::DBI - DBI interface for layer 1.

=head1 SYNOPSIS

Synopsis

=head1 DESCRIPTION

DBI interface for layer 1

=cut

use strict;

use vars qw(@ISA $Driver $ServerHost $DBName);

require DBI;
require MTk::DB;
require MTk::MemberMixin;

@ISA = qw(MTk::DB MTk::MemberMixin);

sub new
{
    my($class) = @_;
    my($self);

    $self = MTk::DB->new();
#    print "got self=$self\n";
    bless($self, $class);

    return $self;
}

sub open
{
    my($self, $dbhost, $dbname) = @_;

#    print "DBI: $ServerHost $DBName $Driver\n";

    $dbhost = $ServerHost unless $dbhost;
    $dbname = $DBName unless $dbname;

    $self->drh(DBI->install_driver($Driver));
#    print "drh=", $self->drh, "\n";

    if (!$self->drh)
    {
	die "DBI Driver install ($Driver) failed: $DBI::err\n";
    }

    $self->dbh($self->drh->connect($dbhost, $dbname));

    if (!$self->dbh)
    {
	die "DBI DB connect failed (host=$dbhost db=$dbname): $DBI::err\n";
    }

    return $self;
}

#
# Perform a query. Snag all the results and initialize a
# MTk::DBQuery object with them.
#

sub query
{
    my($self, $query, @vars) = @_;
    my($q, @rows, $result, @a);

#    print "query: $query\n";
    $result = $self->dbh->do($query);


    if (!defined($result))
    {
	print "query failure: $DBI::errstr\n";
	return undef;
    }

    while (@a = $result->fetchrow)
    {
#	print "got @a\n";
	push(@rows, [@a]);
    }

    if (@rows)
    {
	$q = MTk::DBQuery->new($query, \@vars, \@rows);
    }
    else
    {
	$q = undef;
    }
    return $q;
}

sub error
{
    return $DBI::errstr;
}

#
# Does this table exist?
#
sub table_exists
{
    my($self, $table) = @_;
    my($ans);

    $ans = $self->dbh->func($table, '_ListFields');

    if (!$ans)
    {
#	print "failed: $DBI::errstr\n";
	return undef;
    }
    my($nf) = $ans->{NUMFIELDS};

    return $nf > 0;
}

#
# Clean up.
#

sub DESTROY
{
    my($self) = @_;
    $self->dbh->disconnect();
}

#
# Accessors.
#

sub drh { shift->_elem('drh', @_); }
sub dbh { shift->_elem('dbh', @_); }


1;
